@@ -1,3 +1,4 @@
+import logging
 import numpy as np
 import pandas as pd
 import random
@@ -268,20 +269,34 @@ def add_int_column_from_distribution(dataframe: pd.DataFrame, column_name: str,
 def realign_with_totals(
         dataframe: pd.DataFrame,
         column: str,
-        expectation: int
+        expectation: int,
+        minimum_value: int
 ):
-    if dataframe[column].sum() != expectation:
+    # Create an expanding margin to avoid infinite while
+    margin = 0
+    while dataframe[column].sum() < (expectation - margin) or dataframe[column].sum() > (expectation + margin):
+        logging.info(f"Rebalancing column {column}.  Existing total: {dataframe[column].sum()}, Expectation: "
+                     f"{expectation}, Margin: {margin}")
+        # Get the total outstanding difference
         difference = expectation - dataframe[column].sum()
+        # Pick a number of records to redistribute to
         redistribution_records = dataframe.sample(
             n=abs(difference),
             weights=dataframe[column],
-            replace=True)
+            replace=True
+        )
         redistribution_indices = list(redistribution_records.index.values)
+        # Create a dataframe with redistribution values
         df_update = pd.DataFrame({"index": redistribution_indices})
         df_update["correction"] = 1 if difference > 0 else -1
         df_update = df_update.groupby("index").sum()
+        # Join to original table and add corrections
         dataframe = dataframe.merge(df_update, how="left", left_index=True, right_index=True)
         dataframe["correction"] = dataframe["correction"].fillna(0).astype(int)
         dataframe[column] = dataframe[column] + dataframe["correction"]
+        # Handle minimum values
+        mask = dataframe[column] < minimum_value
+        dataframe.loc[mask, column] = minimum_value
+        dataframe = dataframe.drop(columns=["correction"])
 
-    return dataframe.drop(columns=["correction"])
+    return dataframe
